-- Author: Auz
-- Date: 5.22.2010
-- Version 1.0b
-- Revision Date: 5.28.2010

-- Setting up our Frame

local f = CreateFrame("Frame", "FrostWatchFrame", UIParent)
local menuFrame = CreateFrame("Frame", "FrostWatchMenuFrame", FrostWatchFrame, "UIDropDownMenuTemplate")
local ff = CreateFrame("Slider", "fwOpacitiy", FrostWatchFrame, "OptionsSliderTemplate")

ff:SetMinMaxValues(0.25, 1.0)
ff:SetValueStep(0.05)
ff:SetOrientation("HORIZONTAL")

ff:SetPoint("BOTTOM", FrostWatchFrame,  0, -16)

ff.tooltipText = "Adjust the displays opacity."
getglobal(ff:GetName() .. 'Low'):SetText("")
getglobal(ff:GetName() .. 'High'):SetText("")
getglobal(ff:GetName() .. 'Text'):SetText("") 

ff:Hide()

local fTxt = f:CreateFontString("fwTxt")
f:SetPoint("Center", UIParent, "Center", -200, 200)

f:SetFrameStrata("BACKGROUND")
fTxt:SetJustifyH("LEFT")
fTxt:SetJustifyV("CENTER")

-----------------------------------

-- Our Events to watch for

f:RegisterEvent("ADDON_LOADED")

 ----------------------------------
 
-- Locals

local fwLoaded = false
local realm = GetRealmName()
local player = UnitName("player")
local playerLevel = UnitLevel("player")
local msg = ""
-----------------------------------

-- Check the font size and make sure it's not alpha

local function IsValidSize(option)
	local size = tonumber(option)
	for i=10, 50 do
		if(size == nil) then return false -- they entered something other than a number value
		elseif(size < i or size > 50) then 
			return false 
		elseif(size == i) then 
			return true 
		end
	end
end

-----------------------------------
local function fw_UpdateDisplay(msg)
	fTxt:SetText(msg)
	f:SetWidth(fTxt:GetStringWidth())
	f:SetHeight(fTxt:GetStringHeight())
	fTxt:SetAllPoints(f)
	ff:SetHeight(15)
	ff:SetWidth(f:GetWidth())
end

local function setFontSize(size)
	fwDB.Settings.fontSize = size
	fTxt:SetFont(GameFontNormal:GetFont(), size, "THIN")
	fw_UpdateDisplay(msg)
end

-- Setup the slash command

SLASH_FROSTWATCH1 = "/fw"

function SlashCmdList.FROSTWATCH(msg, editbox)
	local command, option = msg:match("^(%S*)%s*(.-)$")
	if command == "show" then
		if(f:IsVisible()) then
			UIErrorsFrame:AddMessage("FrostWatch is already visible! If you can not see it, try '/shw reset'")
		else 
			fwDB.Settings.visible = true
			f:Show() 
		end
	elseif command == "hide" then
		if(not f:IsVisible()) then
			UIErrorsFrame:AddMessage("FrostWatch is already hidden!")
		else 
			fwDB.Settings.visible = false
			f:Hide() 
		end
	elseif command == "fontsize" and option ~= "" then
		if(IsValidSize(option)) then
			setFontSize(option)
		else UIErrorsFrame:AddMessage("Font Size must be between 10 and 50.") end
	elseif(command == "lock") then
		fwDB.Settings.enableDrag = false
	elseif(command == "unlock") then
		fwDB.Settings.enableDrag = true
	elseif(command == "reset") then
		f:ClearAllPoints()
		f:SetPoint("Center", UIParent, "Center", 0, 0)
	else
		print("FrostWatch Command Syntax:")
		print("--------------------------")
		print("/fw show - Show the display")
		print("/fw hide - Hide the display")
		print("/fw fontsize (10-50) - Set the font size")
		print("/fw lock - Lock the display")
		print("/fw unlock - Unlock the display")
		print("/fw reset - Reset the display")
	end
end

-----------------------------------

-- Get players Emblems of Frost

local function GetPlayerFrost()
	local listSize = GetCurrencyListSize()	
	for i = 1, listSize do
		local name, _, _, _, _, count, _, _, _ = GetCurrencyListInfo(i)
		if(name == "Emblem of Frost") then return count; end
	end
end

-----------------------------------

-- Get all Emblems of Frost from all recorded characters on current realm

local function GetAllFrost()
	local total = 0
	for k,v in pairs(fwDB[realm]) do
		if(v > 0) then
			total = total + v
		end
	end
	return total
end

-----------------------------------

-- Create our player listing to display

local function CreatePlayerListing()
	local msg = "|cffaaaa00Emblems of Frost\nRealm: "..realm..": "..GetAllFrost().."|r"
	for k,v in pairs(fwDB[realm]) do
		if(v > 0) then
			msg = msg.."\n - "..k..": "..v
		end
	end
	return msg
end

-----------------------------------

local function fw_OnEvent(self, event, ...)
	if(event == "ADDON_LOADED" and arg1 == "FrostWatch") then
		print(arg1.." loaded... type /fw for options.")
		if(not fwDB) then fwDB = {} end
		if(not fwDB[realm]) then -- if Realm sub-table doesn't exist, create it.
			fwDB[realm] = {} 
		end
		if(not fwDB[realm][player]) then -- if the Player for the current realm hasn't been recorded yet, record them and set frost to 0 
			if(playerLevel >= 80) then
				fwDB[realm][player] = 0  -- GetPlayerFrost() returns nil at this stage so can't use it here.
			end
		end
		
		if(not fwDB["Settings"]) then fwDB["Settings"] = {} end
		
		if(not fwDB["Settings"]["enableDrag"]) then
			fwDB["Settings"]["enableDrag"] = true
		end
		if(not fwDB["Settings"]["fontSize"]) then
			fwDB["Settings"]["fontSize"] = 16
		end
		if(not fwDB["Settings"]["visible"]) then
			fwDB["Settings"]["visible"] = true
		end
		if(not fwDB["Settings"]["opacity"]) then
			fwDB["Settings"]["opacity"] = 1.0
		end
		ff:SetValue(fwDB.Settings.opacity)
		fTxt:SetFont(GameFontNormal:GetFont(), fwDB.Settings.fontSize, "THIN")
		
		if(fwDB.Settings.visible) then f:Show() else f:Hide() end
		
		msg = "Loading emblems..."
		fw_UpdateDisplay(msg)
		
		fwLoaded = true
		self.TimeSinceLastUpdate = 0
	end
end

fw_UpdateInterval = 5.0; -- How often the OnUpdate code will run (in seconds)

function fw_OnUpdate(self, elapsed)
	fwDB.Settings.opacity = ff:GetValue()
	f:SetAlpha(fwDB.Settings.opacity)
	if(fwLoaded) then
		self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed; 	
		if (self.TimeSinceLastUpdate > fw_UpdateInterval) then
			fwDB[realm][player] = GetPlayerFrost()			
			msg = CreatePlayerListing()
			fw_UpdateDisplay(msg)
			self.TimeSinceLastUpdate = 0;
		end
	end
end

-- Setup our menu
local menu = {
	{ text = "FrostWatch", isTitle = true},
	{ text = "Lock/Unlock Display", func = function() 
												if(fwDB.Settings.enableDrag) then
													fwDB.Settings.enableDrag = false
													print("FrostWatch display locked.")
												else 
													fwDB.Settings.enableDrag = true 
													print("FrostWatch display unlocked.")
												end 
											end },
	{ text = "Hide the Display", func = function() f:Hide(); end },
	{ text = "Show/Hide Opacity Slider", func = function() if(ff:IsVisible()) then ff:Hide(); else ff:Show(); end end },
	{ text = "Font Size", hasArrow = true,
		menuList = {
			{ text = "10", func = function() setFontSize(10); end },
			{ text = "12", func = function() setFontSize(12); end },
			{ text = "14", func = function() setFontSize(14); end },
			{ text = "16", func = function() setFontSize(16); end },
			{ text = "18", func = function() setFontSize(18); end },
			{ text = "20", func = function() setFontSize(20); end },
			{ text = "22", func = function() setFontSize(22); end },
			{ text = "24", func = function() setFontSize(24); end },
			{ text = "26", func = function() setFontSize(26); end },
			{ text = "28", func = function() setFontSize(28); end },
			{ text = "30", func = function() setFontSize(30); end },
		} 
	}
}

local function fw_OnMouseDown(self, button)
	if(button == "RightButton") then
		EasyMenu(menu, menuFrame, "cursor", 0 , 0, "MENU");
	elseif(button == "LeftButton") then
		if(fwDB.Settings.enableDrag) then
			f:StartMoving()
		else print("Display is locked, unlock the frame to move it.") end
	end
end

local function fw_OnMouseUp(self, button)
	if(button == "LeftButton") then
		f:StopMovingOrSizing()
	end
end

f:EnableMouse(true)
f:SetMovable()
f:SetClampedToScreen(true)
f:RegisterForDrag("LeftButton")

f:SetScript("OnEvent", fw_OnEvent)
f:SetScript("OnUpdate", fw_OnUpdate)
f:SetScript("OnMouseDown", fw_OnMouseDown)
f:SetScript("OnMouseUp", fw_OnMouseUp)

