-- Author: Auz
-- Date: 5.22.2010
-- Version 1.0b
-- Revision Date: 5.28.2010

-- Setting up our Frame

local f = CreateFrame("Frame", "ShardWatchFrame", UIParent)
local menuFrame = CreateFrame("Frame", "ShardWatchMenuFrame", ShardWatchFrame, "UIDropDownMenuTemplate")
local sf = CreateFrame("Slider", "swOpacitiy", ShardWatchFrame, "OptionsSliderTemplate")

sf:SetMinMaxValues(0.25, 1.0)
sf:SetValueStep(0.05)
sf:SetOrientation("HORIZONTAL")

sf:SetPoint("BOTTOM", ShardWatchFrame,  0, -16)

sf.tooltipText = "Adjusts the displays opacity."
getglobal(sf:GetName() .. 'Low'):SetText("")
getglobal(sf:GetName() .. 'High'):SetText("")
getglobal(sf:GetName() .. 'Text'):SetText("") 

sf:Hide()

local fTxt = f:CreateFontString("swTxt")
f:SetPoint("Center", UIParent, "Center", -200, 200)

f:SetFrameStrata("BACKGROUND")
fTxt:SetJustifyH("LEFT")
fTxt:SetJustifyV("CENTER")

-----------------------------------

-- Our Events to watch for

f:RegisterEvent("ADDON_LOADED")

 ----------------------------------
 
-- Locals

local swLoaded = false
local realm = GetRealmName()
local player = UnitName("player")
local playerLevel = UnitLevel("player")
local msg = ""
-----------------------------------

-- Check the font size and make sure it's not alpha

local function IsValidSize(option)
	local size = tonumber(option)
	for i=10, 50 do
		if(size == nil) then return false -- they entered something other than a number value
		elseif(size < i or size > 50) then 
			return false 
		elseif(size == i) then 
			return true 
		end
	end
end

-----------------------------------
local function sw_UpdateDisplay(msg)
	fTxt:SetText(msg)
	f:SetWidth(fTxt:GetStringWidth())
	f:SetHeight(fTxt:GetHeight())
	fTxt:SetAllPoints(f)
	sf:SetHeight(15)
	sf:SetWidth(f:GetWidth())
end

local function setFontSize(size)
	shardDB.Settings.fontSize = size
	fTxt:SetFont(GameFontNormal:GetFont(), size, "THIN")
	sw_UpdateDisplay(msg)
end

-- Setup the slash command

SLASH_SHARDWATCH1 = "/shw"

function SlashCmdList.SHARDWATCH(msg, editbox)
	local command, option = msg:match("^(%S*)%s*(.-)$")
	if command == "show" then
		if(f:IsVisible()) then
			UIErrorsFrame:AddMessage("ShardWatch is already visible! If you can not see it, try '/shw reset'")
		else 
			shardDB.Settings.visible = true
			f:Show() 
		end
	elseif command == "hide" then
		if(not f:IsVisible()) then
			UIErrorsFrame:AddMessage("ShardWatch is already hidden!")
		else 
			shardDB.Settings.visible = false
			f:Hide() 
		end
	elseif command == "fontsize" and option ~= "" then
		if(IsValidSize(option)) then
			setFontSize(option)
		else UIErrorsFrame:AddMessage("Font Size must be between 10 and 50.") end
	elseif(command == "lock") then
		shardDB.Settings.enableDrag = false
	elseif(command == "unlock") then
		shardDB.Settings.enableDrag = true
	elseif(command == "reset") then
		f:ClearAllPoints()
		f:SetPoint("Center", UIParent, "Center", 0, 0)
	else
		print("ShardWatch Command Syntax:")
		print("--------------------------")
		print("/shw show - Show the display")
		print("/shw hide - Hide the display")
		print("/shw fontsize (10-50) - Set the font size")
		print("/shw lock - Lock the display")
		print("/shw unlock - Unlock the display")
		print("/shw reset - Reset the display")
	end
end

-----------------------------------

-- Get players Stone Keeper's Shards

local function GetPlayerShards()
	local listSize = GetCurrencyListSize()	
	for i = 1, listSize do
		local name, _, _, _, _, count, _, _, _ = GetCurrencyListInfo(i)
		if(name == "Stone Keeper's Shard") then return count; end
	end
end

-----------------------------------

-- Get all Stone Keeper's Shards from all recorded characters on current realm

local function GetAllShards()
	local total = 0
	for k,v in pairs(shardDB[realm]) do
		if(v > 0) then
			total = total + v
		end
	end
	return total
end

-----------------------------------

-- Create our player listing to display

local function CreatePlayerListing()
	local msg = "|cffaaaa00Stone Keeper's Shards\nRealm: "..realm..": "..GetAllShards().."|r"
	for k,v in pairs(shardDB[realm]) do
		if(v > 0) then
			msg = msg.."\n - "..k..": "..v
		end
	end
	return msg
end

-----------------------------------

local function sw_OnEvent(self, event, ...)
	if(event == "ADDON_LOADED" and arg1 == "ShardWatch") then
		print(arg1.." loaded... type /shw for options.")
		if(not shardDB) then shardDB = {} end
		if(not shardDB[realm]) then -- if Realm sub-table doesn't exist, create it.
			shardDB[realm] = {} 
		end
		if(not shardDB[realm][player]) then -- if the Player for the current realm hasn't been recorded yet, record them and set frost to 0 
			if(playerLevel >= 70) then   	-- if the player isn't 80, don't record them as they probably don't have any shards yet.
				shardDB[realm][player] = 0  -- GetPlayerShards() returns nil at this stage so can't use it here.
			end
		end
		
		if(not shardDB["Settings"]) then shardDB["Settings"] = {} end
		
		if(not shardDB["Settings"]["enableDrag"]) then
			shardDB["Settings"]["enableDrag"] = true
		end
		if(not shardDB["Settings"]["fontSize"]) then
			shardDB["Settings"]["fontSize"] = 16
		end
		if(not shardDB["Settings"]["visible"]) then
			shardDB["Settings"]["visible"] = true
		end
		if(not shardDB["Settings"]["opacity"]) then
			shardDB["Settings"]["opacity"] = 1.0
		end
		sf:SetValue(shardDB.Settings.opacity)
		fTxt:SetFont(GameFontNormal:GetFont(), shardDB.Settings.fontSize, "THIN")
		
		if(shardDB.Settings.visible) then f:Show() else f:Hide() end
		
		msg = "Loading shards..."
		sw_UpdateDisplay(msg)

		swLoaded = true
		self.TimeSinceLastUpdate = 0
	end
end

sw_UpdateInterval = 5.0; -- How often emblems/shards are updated (in seconds)

function sw_OnUpdate(self, elapsed)
	shardDB.Settings.opacity = sf:GetValue()
	f:SetAlpha(shardDB.Settings.opacity)
	if(swLoaded) then
		self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed; 	
		if (self.TimeSinceLastUpdate > sw_UpdateInterval) then
			shardDB[realm][player] = GetPlayerShards()			
			msg = CreatePlayerListing()
			sw_UpdateDisplay(msg)
			self.TimeSinceLastUpdate = 0;
		end
	end
end

-- Setup our menu
local menu = {
	{ text = "ShardWatch", isTitle = true},
	{ text = "Lock/Unlock Display", func = function() 
												if(shardDB.Settings.enableDrag) then
													shardDB.Settings.enableDrag = false
													print("ShardWatch display locked.")
												else 
													shardDB.Settings.enableDrag = true 
													print("ShardWatch display unlocked.")
												end 
											end },
	{ text = "Hide the Display", func = function() f:Hide(); end },
	{ text = "Show/Hide Opacity Slider", func = function() if(sf:IsVisible()) then sf:Hide(); else sf:Show(); end end },
	{ text = "Font Size", hasArrow = true,
		menuList = {
			{ text = "10", func = function() setFontSize(10); end },
			{ text = "12", func = function() setFontSize(12); end },
			{ text = "14", func = function() setFontSize(14); end },
			{ text = "16", func = function() setFontSize(16); end },
			{ text = "18", func = function() setFontSize(18); end },
			{ text = "20", func = function() setFontSize(20); end },
			{ text = "22", func = function() setFontSize(22); end },
			{ text = "24", func = function() setFontSize(24); end },
			{ text = "26", func = function() setFontSize(26); end },
			{ text = "28", func = function() setFontSize(28); end },
			{ text = "30", func = function() setFontSize(30); end },
		} 
	}
}

local function sw_OnMouseDown(self, button)
	if(button == "RightButton") then
		EasyMenu(menu, menuFrame, "cursor", 0 , 0, "MENU");
	elseif(button == "LeftButton") then
		if(shardDB.Settings.enableDrag) then
			f:StartMoving()
		else print("Display is locked, unlock the frame to move it.") end
	end
end

local function sw_OnMouseUp(self, button)
	if(button == "LeftButton") then
		f:StopMovingOrSizing()
	end
end

f:EnableMouse(true)
f:SetMovable()
f:SetClampedToScreen(true)
f:RegisterForDrag("LeftButton")

f:SetScript("OnEvent", sw_OnEvent)
f:SetScript("OnUpdate", sw_OnUpdate)
f:SetScript("OnMouseDown", sw_OnMouseDown)
f:SetScript("OnMouseUp", sw_OnMouseUp)

