-- Author: Auz
-- Date: 5.22.2010
-- Version 1.0b
-- Revision Date: 5.28.2010

-- Setting up our Frame

local f = CreateFrame("Frame", "TriumphWatchFrame", UIParent)
local menuFrame = CreateFrame("Frame", "TriumphWatchMenuFrame", TriumphWatchFrame, "UIDropDownMenuTemplate")
local tf = CreateFrame("Slider", "twOpacitiy", TriumphWatchFrame, "OptionsSliderTemplate")

tf:SetMinMaxValues(0.25, 1.0)
tf:SetValueStep(0.05)
tf:SetOrientation("HORIZONTAL")

tf:SetPoint("BOTTOM", TriumphWatchFrame,  0, -16)

tf.tooltipText = "Adjust the displays opacity."
getglobal(tf:GetName() .. 'Low'):SetText("")
getglobal(tf:GetName() .. 'High'):SetText("")
getglobal(tf:GetName() .. 'Text'):SetText("") 

tf:Hide()

local fTxt = f:CreateFontString("twTxt")
f:SetPoint("Center", UIParent, "Center", -200, 200)

f:SetFrameStrata("BACKGROUND")
fTxt:SetJustifyH("LEFT")
fTxt:SetJustifyV("CENTER")

-----------------------------------

-- Our Events to watch for

f:RegisterEvent("ADDON_LOADED")

 ----------------------------------
 
-- Locals

local twLoaded = false
local realm = GetRealmName()
local player = UnitName("player")
local playerLevel = UnitLevel("player")
local msg = ""
-----------------------------------

-- Check the font size and make sure it's not alpha

local function IsValidSize(option)
	local size = tonumber(option)
	for i=10, 50 do
		if(size == nil) then return false -- they entered something other than a number value
		elseif(size < i or size > 50) then 
			return false 
		elseif(size == i) then 
			return true 
		end
	end
end

-----------------------------------
local function tw_UpdateDisplay(msg)
	fTxt:SetText(msg)
	f:SetWidth(fTxt:GetStringWidth())
	f:SetHeight(fTxt:GetStringHeight())
	fTxt:SetAllPoints(f)
	tf:SetHeight(15)
	tf:SetWidth(f:GetWidth())
end

local function setFontSize(size)
	twDB.Settings.fontSize = size
	fTxt:SetFont(GameFontNormal:GetFont(), size, "THIN")
	tw_UpdateDisplay(msg)
end

-- Setup the slash command

SLASH_TRIUMPHWATCH1 = "/tw"

function SlashCmdList.TRIUMPHWATCH(msg, editbox)
	local command, option = msg:match("^(%S*)%s*(.-)$")
	if command == "show" then
		if(f:IsVisible()) then
			UIErrorsFrame:AddMessage("TriumphWatch is already visible! If you can not see it, try '/tw reset'")
		else 
			twDB.Settings.visible = true
			f:Show() 
		end
	elseif command == "hide" then
		if(not f:IsVisible()) then
			UIErrorsFrame:AddMessage("TriumphWatch is already hidden!")
		else 
			twDB.Settings.visible = false
			f:Hide() 
		end
	elseif command == "fontsize" and option ~= "" then
		if(IsValidSize(option)) then
			setFontSize(option)
		else UIErrorsFrame:AddMessage("Font Size must be between 10 and 50.") end
	elseif(command == "lock") then
		twDB.Settings.enableDrag = false
	elseif(command == "unlock") then
		twDB.Settings.enableDrag = true
	elseif(command == "reset") then
		f:ClearAllPoints()
		f:SetPoint("Center", UIParent, "Center", 0, 0)
	else
		print("TriumphWatch Command Syntax:")
		print("--------------------------")
		print("/tw show - Show the display")
		print("/tw hide - Hide the display")
		print("/tw fontsize (10-50) - Set the font size")
		print("/tw lock - Lock the display")
		print("/tw unlock - Unlock the display")
		print("/tw reset - Reset the display")
	end
end

-----------------------------------

-- Get players Emblems of Triumph

local function GetPlayerTriumph()
	local listSize = GetCurrencyListSize()	
	for i = 1, listSize do
		local name, _, _, _, _, count, _, _, _ = GetCurrencyListInfo(i)
		if(name == "Emblem of Triumph") then return count; end
	end
end

-----------------------------------

-- Get all Emblems of Triumph from all recorded characters on current realm

local function GetAllTriumph()
	local total = 0
	for k,v in pairs(twDB[realm]) do
		if(v > 0) then
			total = total + v
		end
	end
	return total
end

-----------------------------------

-- Create our player listing to display

local function CreatePlayerListing()
	local msg = "|cffaaaa00Emblems of Triumph\nRealm: "..realm..": "..GetAllTriumph().."|r"
	for k,v in pairs(twDB[realm]) do
		if(v > 0) then -- if 0 then don't list
			msg = msg.."\n - "..k..": "..v
		end
	end
	return msg
end

-----------------------------------

local function tw_OnEvent(self, event, ...)
	if(event == "ADDON_LOADED" and arg1 == "TriumphWatch") then
		print(arg1.." loaded... type /tw for options.")
		if(not twDB) then twDB = {} end
		if(not twDB[realm]) then -- if Realm sub-table doesn't exist, create it.
			twDB[realm] = {} 
		end
		if(not twDB[realm][player]) then -- if the Player for the current realm hasn't been recorded yet, record them and set frost to 0 
			if(playerLevel >= 70) then   	-- if the player isn't 80, don't record them as they probably don't have any shards yet.
				twDB[realm][player] = 0  -- GetPlayerTriumph() returns nil at this stage so can't use it here.
			end
		end
		
		if(not twDB["Settings"]) then twDB["Settings"] = {} end
		
		if(not twDB["Settings"]["enableDrag"]) then
			twDB["Settings"]["enableDrag"] = true
		end
		if(not twDB["Settings"]["fontSize"]) then
			twDB["Settings"]["fontSize"] = 16
		end
		if(not twDB["Settings"]["visible"]) then
			twDB["Settings"]["visible"] = true
		end
		if(not twDB["Settings"]["opacity"]) then
			twDB["Settings"]["opacity"] = 1.0
		end
		tf:SetValue(twDB.Settings.opacity)
		fTxt:SetFont(GameFontNormal:GetFont(), twDB.Settings.fontSize, "THIN")
		
		if(twDB.Settings.visible) then f:Show() else f:Hide() end
		
		msg = "Loading emblems..."
		tw_UpdateDisplay(msg)
		
		twLoaded = true
		self.TimeSinceLastUpdate = 0
	end
end

tw_UpdateInterval = 5.0; -- How often the OnUpdate code will run (in seconds)

function tw_OnUpdate(self, elapsed)
	twDB.Settings.opacity = tf:GetValue()
	f:SetAlpha(twDB.Settings.opacity)
	if(twLoaded) then
		self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed; 	
		if (self.TimeSinceLastUpdate > tw_UpdateInterval) then
			twDB[realm][player] = GetPlayerTriumph()			
			msg = CreatePlayerListing()
			tw_UpdateDisplay(msg)
			self.TimeSinceLastUpdate = 0;
		end
	end
end

-- Setup our menu
local menu = {
	{ text = "TriumphWatch", isTitle = true},
	{ text = "Lock/Unlock Display", func = function() 
												if(twDB.Settings.enableDrag) then
													twDB.Settings.enableDrag = false
													print("TriumphWatch display locked.")
												else 
													twDB.Settings.enableDrag = true 
													print("TriumphWatch display unlocked.")
												end 
											end },
	{ text = "Hide the Display", func = function() f:Hide(); end },
	{ text = "Show/Hide Opacity Slider", func = function() if(tf:IsVisible()) then tf:Hide(); else tf:Show(); end end },
	{ text = "Font Size", hasArrow = true,
		menuList = {
			{ text = "10", func = function() setFontSize(10); end },
			{ text = "12", func = function() setFontSize(12); end },
			{ text = "14", func = function() setFontSize(14); end },
			{ text = "16", func = function() setFontSize(16); end },
			{ text = "18", func = function() setFontSize(18); end },
			{ text = "20", func = function() setFontSize(20); end },
			{ text = "22", func = function() setFontSize(22); end },
			{ text = "24", func = function() setFontSize(24); end },
			{ text = "26", func = function() setFontSize(26); end },
			{ text = "28", func = function() setFontSize(28); end },
			{ text = "30", func = function() setFontSize(30); end },
		} 
	}
}

local function tw_OnMouseDown(self, button)
	if(button == "RightButton") then
		EasyMenu(menu, menuFrame, "cursor", 0 , 0, "MENU");
	elseif(button == "LeftButton") then
		if(twDB.Settings.enableDrag) then
			f:StartMoving()
		else print("Display is locked, unlock the frame to move it.") end
	end
end

local function tw_OnMouseUp(self, button)
	if(button == "LeftButton") then
		f:StopMovingOrSizing()
	end
end

f:EnableMouse(true)
f:SetMovable()
f:SetClampedToScreen(true)
f:RegisterForDrag("LeftButton")

f:SetScript("OnEvent", tw_OnEvent)
f:SetScript("OnUpdate", tw_OnUpdate)
f:SetScript("OnMouseDown", tw_OnMouseDown)
f:SetScript("OnMouseUp", tw_OnMouseUp)

